'use strict';

// 以下をコメントアウトするとデバッグ出力が有効になります
//process.env.DEBUG = 'actions-on-google:*';

const functions = require('firebase-functions');
const { dialogflow } = require('actions-on-google');

const app = dialogflow({
  verification: {
    // 認証用ヘッダ
    Authorization: 'Bearer secret',
  },
});

// クイズデータを保存するコンテキスト
const CONTEXT_QUIZ = 'quiz';

// コンテキストに保存されたクイズデータを取得
function getQuizFromContext(conv) {
  const ctx = conv.contexts.get(CONTEXT_QUIZ);
  return ctx ? ctx.parameters.quiz : null;
}

app.intent('Default Welcome Intent', quiz);
app.intent('Quiz', quiz);
app.intent('QuizAnswer - yes', quiz);

// クイズを出題する
function quiz(conv, params, input) {
  const quiz = require('./outsider-quiz').create();
  // quizの中身は以下のようなオブジェクト
  // {
  //   items: [ 'りんご', 'バナナ', '飛行機' ],
  //   answerIndex: 2
  // }

  // コンテキストにデータを保存
  conv.contexts.set(CONTEXT_QUIZ, 1, {
    quiz: quiz,
  });

  conv.ask('<speak>仲間外れはどれでしょう？' +
           '<break time="500ms"/>' +
           quiz.items.join('、') + '</speak>');
}

app.intent('QuizAnswer', (conv, params, input) => {
  // ItemNameパラメータを受け取る
  const itemName = params.ItemName;

  const quiz = getQuizFromContext(conv);
  if (!quiz) {
    // クイズデータがない
    conv.close('クイズを始めるには「クイズ」と言ってください。');
    return;
  }

  // 正解のアイテム名
  const correctAnswer = quiz.items[quiz.answerIndex];

  let reply = '<speak>';
  if (correctAnswer === itemName) {
    reply += '<emphasis level="strong">正解です！</emphasis>';
  } else {
    reply += 'ハズレです。正解は「' + correctAnswer + '」でした。';
  }
  reply += '<break time="500ms"/>もう一回、やりませんか？</speak>';
  conv.ask(reply);
});

app.intent('Quiz - repeat', quizRepeat);
app.intent('Quiz - noinput', quizRepeat);

// 問題文を繰り返す
function quizRepeat(conv, params, input) {
  const quiz = getQuizFromContext(conv);

  conv.ask('<speak>仲間外れはどれでしょう？<break time="500ms"/>' +
           quiz.items.join('、') + '</speak>');
}

app.intent('QuizAnswer - noinput', conv => {
  conv.ask('もう一回、やりませんか？');
});

// quizという関数名でエクスポート
exports.quiz = functions.https.onRequest(app);
